#
# Copyright (C) 2001, 2002 Caldera International, Inc.  All Rights Reserved.
#
# Library routines and global defines for MSG backend scripts
#

use Getopt::Long;
use Net::Domain qw(hostname hostfqdn hostdomain);
use Net::LDAP qw(:all);                      # use for all code
use Net::LDAP::LDIF qw(:all);                # use for all code
use Net::LDAP::Util qw(ldap_error_name       
                       ldap_error_text) ;    # use for Error handling
use IO::Select;
use IO::File;

use Locale::Msgcat;
use File::Basename();
use Text::Iconv();

#
# Globals
#
$errmsg		  = "";
$ldaphost	  = "";
$ldapsuffix 	  = "";
$rootdn  	  = "";
$rootpw  	  = "";
$msgadmin         = "";
$msgadmindn       = "";
$msgadminpw       = "";
$msgroot 	  = "";
$msgbin  	  = "";
$msgdn		  = "";
$msguser	  = "";
$msgpw		  = "";
$msgtmp		  = "";
$msgkey           = "";
$msgsecure        = "";
$msgoutlkmboxtype = "";
$msgctoc          = "";

$msgi18n_tx_to_converter = 0;
$msgi18n_tx_from_converter = 0;

# our message catalog name
$msgi18nfile = "msgutil.cat";

# hard code NLSPATH to avoid security problems
$msgi18n_nlspath = "/usr/share/locale/%L/%N:/usr/share/locale/%L/LC_MESSAGES/%N:/usr/share/locale/%l/%N:/usr/share/locale/%l/LC_MESSAGES/%N:";

# set to 1 to die if message not contain right number of arguments:
$msgi18ndebug = 0;

# list of attributes in the user object we care about
@userattrlist = (
	uid,
	ou,
	givenname,
	sn,
	cn,
	mail,
	calderaMsgAlternateAddress,
	calderaMsgForwardingAddress,
	calderaMsgForwardCopy,
	homePhone,
	telephoneNumber,
	facsimileTelephoneNumber,
	pager,
	mobile,
	title,
	physicalDeliveryOfficeName,
	calderaMsgVacationEnabled,
	calderaMsgVacationSubject,
	calderaMsgVacationMessage,
	calderaMsgSpamFilterEnabled,
	calderaMsgSpamDestroyEnabled,
	calderaMsgSpamAllowed,
	calderaMsgSpamExclude,
	calderaMsgQuotaLimit,
);

# our list of services and their actions.
%msgservices = (
	steltor_cts =>
		{
		service		=> "steltor_cts",
		comm		=> "uniengd",
		description	=> "Steltor Corporate Time Server",
		command		=> "unieng",
		rcommand	=> "/etc/rc.d/init.d/steltor_cts",
		action		=> ["stop", "start", "restart"],
		status		=> "DOWN"
		},
	ldap =>
		{
		service		=> "ldap",
		comm		=> "slapd",
		description	=> "LDAP Directory Server",
		command		=> "slapd",
		rcommand	=> "/etc/rc.d/init.d/ldap",
		action		=> ["restart"],
		status		=> "DOWN"
		},

	postfix =>
		{
		service		=> "postfix",
		comm		=> "master",
		description	=> "Postfix Mail Transport Agent",
		command		=> "/usr/libexec/postfix/master",
		rcommand	=> "/etc/rc.d/init.d/postfix",
		action		=> ["start", "stop", "restart", "reload"],
		status		=> "DOWN"
		},

	cyrus =>
		{
		service		=> "cyrus",
		comm		=> "master",
		description	=> "Cyrus IMAP/POP Server",
		command		=> "/usr/cyrus/bin/master",
		rcommand	=> "/etc/rc.d/init.d/cyrus",
		action		=> ["start", "stop", "restart"],
		status		=> "DOWN"
		},

	docview	=>
		{
		service		=> "docview",
		comm		=> "httpd",
		description	=> "DocView OpenLinux Documentation Server",
		command		=> "docview -d /usr/lib/docview -f /usr/lib/docview/conf/httpd.conf",
		rcommand	=> "/etc/rc.d/init.d/docview",
		action		=> ["start", "stop", "restart", "reload"],
		status		=> "DOWN"
		}
);

# must be initialized by call to msgi18naction
%msgi18naction = (
	start	=> "start",
	stop	=> "stop",
	restart => "restart",
	reload  => "reload",
);

#
# Functions
#

sub
Trim {
	my $string = $_;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

sub
getLDAPerrorString 
{
    my ($from,$mesg) = @_;
    $errmsg = "ERROR:$mesg->code:" .
	$from . " (" . ldap_error_name($mesg->code) . ") " . ldap_error_text($mesg->code);
    return $errmsg;
}

sub
I_am_interactive
{
        if (!system("/usr/bin/tty -s")) {
        	return 1
        } else {
        	return 0
        }
}

sub
parse_conf
{
  my ($noauth) = @_;
  my $ldapport = "";

  open(FD, "/etc/opt/lsb-caldera.com-volution/msg/msg.conf") || die msgi18nsmsg(11, 1, "msg.conf", "$@"); # Unable to open file
  @lines = <FD>;
  close(FD);

 
  foreach(@lines) {
        if (/^LDAPHOST:/) {
                @vals = split(":");
                $ldaphost = $vals[1];
                chomp($ldaphost);
        }
	if (/^LDAPPORT:/) {
                @vals = split(":");
                $ldapport = $vals[1];
                chomp($ldapport);
	}
        if (/^ROOTDN:/) {
                @vals = split(":");
                $rootdn = $vals[1];
                chomp($rootdn);
        }
        if (/^ROOTPW:/) {
                @vals = split(":");
                $rootpw = $vals[1];
                chomp($rootpw);
        }
        if (/^LDAPSUFFIX:/) {
                @vals = split(":");
                $ldapsuffix = $vals[1];
                chomp($ldapsuffix);
        }
        if (/^ADMIN:/) {
                @vals = split(":");
                $msgadmin = $vals[1];
                chomp($msgadmin);
        }
        if (/^ADMINDN:/) {
                @vals = split(":");
                $msgadmindn = $vals[1];
                chomp($msgadmindn);
        }
        if (/^ADMINPW:/) {
                @vals = split(":");
                $msgadminpw = $vals[1];
                chomp($msgadminpw);
        }
        if (/^ROOT:/) {
                @vals = split(":");
                $msgroot = $vals[1];
                chomp($msgroot);
        }
        if (/^BIN:/) {
                @vals = split(":");
                $msgbin = $vals[1];
                chomp($msgbin);
        }
        if (/^DN:/) {
                @vals = split(":");
                $msgdn = $vals[1];
                chomp($msgdn);
        }
        if (/^USER:/) {
                @vals = split(":");
                $msguser = $vals[1];
                chomp($msguser);
        }
        if (/^PW:/) {
                @vals = split(":");
                $msgpw = $vals[1];
                chomp($msgpw);
        }
        if (/^TMP:/) {
                @vals = split(":");
                $msgtmp = $vals[1];
                chomp($msgtmp);
        }
        if (/^VAROPT:/) {
                @vals = split(":");
                $msgvaropt = $vals[1];
                chomp($msgvaropt);
        }
        if (/^SECURE:/) {
                @vals = split(":");
                $msgsecure = $vals[1];
                chomp($msgsecure);
        }
        if (/^OUTLKMBOXTYPE:/) {
                @vals = split(":");
                $msgoutlkmboxtype = $vals[1];
                chomp($msgoutlkmboxtype);
	}
	if (/^CTOC:/) {
		@vals = split(":");
		$msgctoc = $vals[1];
		chomp($msgctoc);
	}	
  }

  # Append the ldap port name to the ldap host name if its set
  if ($ldapport) {
	$ldaphost = "$ldaphost:$ldapport";
  }
  if ($MSGDEBUG) {
	print "ldaphost   = $ldaphost\n";
        print "ldapsuffix = $ldapsuffix\n";
        print "rootdn     = $rootdn\n";
        print "rootpw     = secret \n";
  }

  if ($noauth) {
	return;
  }
  # Swap out the msgdn and set the msgadminpw for those of the user if
  # they were given to stdin.

  if (! -c *STDIN) {
	  ($udn, $upw, $ukey)  = <STDIN>;
	  if ( $udn ) {
		$msgadmindn = $udn;
		chomp($msgadmindn);
		$msgadmin = getfirstfromdn($msgadmindn);
	  }
	  if ( $upw ) {
		$msgadminpw = $upw;
		chomp($msgadminpw);
	  }
	  if ( $ukey ) {
		$msgkey = $ukey;
		chomp($msgkey);
	  }
  }

  # If the password wasn't given to stdin, or came from msg.conf , then 
  # we need to prompt for a password.
  if ( !$msgadminpw ) {
  	my $tty = (IO::File->new('/dev/tty', O_RDWR) ||
                 *STDERR || *STDIN || *STDOUT);
  	$tty->autoflush(1);
  	$tty->print(msgi18nsmsg(2, 1)); # Password prompt:
  	my $ostty;
  	chomp($ostty = `stty -g`);
  	system("/bin/stty -echo -icanon min 1 time 0 2>/dev/null || " .  
		"/bin/stty -echo cbreak");
  	chomp($msgadminpw = $tty->getline);
  	$tty->print("\n");
  	system("/bin/stty $ostty");
  }
}

# this depends on a valid dn, parse and get the first attribute.
sub
getfirstfromdn
{
	my $dn = @_[0];
	my @tmp = split(',', $dn);
	my @first = split('=', $tmp[0]);
	my $val = $first[1];
	return $val;
}

# verify that argument length does not exceed the given value
sub
arglengthcheck
{
	my ($attr, $val, $maxlen) = @_;

	$len = length($val);

	if ($len > $maxlen) {
		print STDERR "ERROR:256:--$attr length exceeds maximum value of $maxlen\n";
		print STDERR msgi18nsmsg(15, 1, $attr, $val);
		exit(1);
	}
}

# uids are allowed characters [a-zA_Z0-9\-_]
sub
check_uid
{
	my ($attr, $val) = @_;

	if ($val !~ m/^[a-zA-Z0-9\-_]*$/) {
		$bad = val;
		$bad =~ s/[a-zA-Z0-9\-_]*//g;
		# $var=$val contains invalid characters: $bad
		print STDERR msgi18nsmsg(16, 1, $attr, $val, $bad);
		exit(1);
	}
}

# mail parts are on either side of the '@' character in a mail address
# and are allowed characters [a-zA_A0-9\-_\.]
sub
check_mailpart
{
	my ($attr, $val) = @_;

	if ($val !~ m/^[a-zA-Z0-9\-_\.]*$/) {
		$bad = $val;
		$bad =~ s/[a-zA-Z0-9\-_\.]*//g;
		# $var=$val contains invalid characters: $bad
		print STDERR msgi18nsmsg(16, 1, $attr, $val, $bad);
		exit(1);
	}
}

# phone numbers in slapd are alphanumeric plus a few other characters
# [a-zA-Z0-9\(\)\+,\-\.\/:\? ].
# = is also allowed by slapd but we do not escape it
# so it does not work currently.
sub
check_phone
{
	my ($attr, $val) = @_;

	if ($val !~ m/^[a-zA-Z0-9\(\)\+,\-\.\/:\? ]*$/) {
		$bad = $val;
		$bad =~ s/[a-zA-Z0-9\(\)\+,\-\.\/:\? ]*//g;
		# $var=$val contains invalid characters: $bad
		print STDERR msgi18nsmsg(16, 1, $attr, $val, $bad);
		exit(1);
	}
}

# valid mail address are mailpart@mailpart
# neither mailpart can be null
sub
check_mail
{
	my ($attr, $val) = @_;

	my @tmp = split('@', $val);
	if (scalar(@tmp) != 2) {
		# syntax error in mail address: --$attr=$val\n
		print STDERR msgi18nsmsg(13, 1, $attr, $val);
		exit(1);
	}
	my $name = $tmp[0];
	my $domain = $tmp[1];
	if ($name eq "") {
		# syntax error in mail address: --$attr=$val\n
		print STDERR msgi18nsmsg(13, 1, $attr, $val);
		exit(1);
	}
	if  ($domain eq "") {
		# syntax error in mail address: --$attr=$val\n
		print STDERR msgi18nsmsg(13, 1, $attr, $val);
		exit(1);
	}
	check_mailpart($attr, $name);
	check_mailpart($attr, $domain);
}

# this routine's primary purpose is to translate into utf8 for storage
# into the ldap server, we reject control characters only.
sub
check_utf
{
	my ($attr, $val) = @_;

	my $val = msgi18n_tx_to_utf8($val);
	if (!$val) {
		# $var=$val contains invalid characters: $bad
		$bad = "";
		print STDERR msgi18nsmsg(16, 1, $attr, $val, $bad);
		exit(1);
	}
	# translate in place the value above.
	@_[1] = $val;
	if ($val !~ m/^[ -\377]*$/) {
		$bad = $val;
		$bad =~ s/[ -\377]*//g;
		# $var=$val contains invalid characters: $bad
		print STDERR msgi18nsmsg(16, 1, $attr, $val, $bad);
		exit(1);
	}
}

# mailstrings are used to match mail addresses and can be any part thereof
# for this check we all all the characters in a mail address but do
# no insist on a single '@' character so we allow the charset [a-zA-Z0-9_\.@]
sub
check_mailstring
{
	my ($attr, $val) = @_;

	if ($val !~ m/^[a-zA-Z0-9\-_\.@]*$/) {
		$bad = $val;
		$bad =~ s/[a-zA-Z0-9\-_\.@]*//g;
		# $var=$val contains invalid characters: $bad
		print STDERR msgi18nsmsg(16, 1, $attr, $val, $bad);
		exit(1);
	}
}

#
# check that a path is valid, this is a syntax check and for now
# is just has to start with a '/'
sub
check_filename
{
	my ($attr, $val) = @_;

	if (substr($val, 0, 1) ne "/") {
		# --$attr=$val is not a fully qualified pathname\n
		print STDERR msgi18nsmsg(14, 1, $attr, $val);
		exit(1);
	}
}

# check for digits or NONE.
sub
check_quota
{
	my ($attr, $val) = @_;

	if ($val eq "NONE") {
		return;
	}
	if ($val !~ m/^[0-9]*$/) {
		$bad = $val;
		$bad =~ s/[0-9]*//g;
		# $var=$val contains invalid characters: $bad
		print STDERR msgi18nsmsg(16, 1, $attr, $val, $bad);
		exit(1);
	}
}

# this is our global argument checker
# is is concerned with an ever growing set of checks including
# mail address syntax, syntax,
# and character set, and argument length.
sub
argcheck
{
	my ($attr, $val) = @_;

	# only check arguments with a value
	if ($val eq "") {
		return;
	}
	if ($attr eq "vacationmessage") {
		$newline = 1;
		arglengthcheck($attr, $val, 1024);
	} else {
		arglengthcheck($attr, $val, 128);
	}
	if ($attr eq "uid") {
		check_uid($attr, $val);
	}
	if ($attr eq "mail") {
		check_mail($attr, $val);
	}
	if ($attr eq "name") {
		check_mailpart($attr, $val);
	}
	if ($attr eq "domain") {
		check_mailpart($attr, $val);
	}
	if ($attr eq "olddomain") {
		check_mailpart($attr, $val);
	}
	if ($attr eq "newdomain") {
		check_mailpart($attr, $val);
	}
	if ($attr eq "owner") {
		check_mail($attr, $val);
	}
	if ($attr eq "owneruid") {
		check_uid($attr, $val);
	}
	if ($attr eq "member") {
		check_mail($attr, $val);
	}
	if ($attr eq "memberuid") {
		check_uid($attr, $val);
	}
	if ($attr eq "memberalias") {
		check_mailpart($attr, $val);
	}
	if ($attr eq "file") {
		check_filename($attr, $val);
	}
	if ($attr eq "program") {
		check_filename($attr, $val);
	}
	if ($attr eq "remote") {
		check_mail($attr, $val);
	}
	if ($attr eq "description") {
		check_utf($attr, $_[1]);
	}
	if ($attr eq "firstname") {
		check_utf($attr, $_[1]);
	}
	if ($attr eq "lastname") {
		check_utf($attr, $_[1]);
	}
	if ($attr eq "fullname") {
		check_utf($attr, $_[1]);
	}
	if ($attr eq "homephone") {
		check_phone($attr, $val);
	}
	if ($attr eq "businessphone") {
		check_phone($attr, $val);
	}
	if ($attr eq "fax") {
		check_phone($attr, $val);
	}
	if ($attr eq "pager") {
		check_phone($attr, $val);
	}
	if ($attr eq "mobile") {
		check_phone($attr, $val);
	}
	if ($attr eq "title") {
		check_utf($attr, $_[1]);
	}
	if ($attr eq "mailstop") {
		check_utf($attr, $_[1]);
	}
	if ($attr eq "forward") {
		check_mail($attr, $val);
	}
	if ($attr eq "alternatemail") {
		check_mail($attr, $val);
	}
	if ($attr eq "vacationsubject") {
		# no checks, binary allowed.
	}
	if ($attr eq "vacationmessage") {
		# no checks, binary allowed.
	}
	if ($attr eq "spamallowed") {
		check_mailstring($attr, $val);
	}
	if ($attr eq "delallowed") {
		check_mailstring($attr, $val);
	}
	if ($attr eq "spamexclude") {
		check_mailstring($attr, $val);
	}
	if ($attr eq "delexclude") {
		check_mailstring($attr, $val);
	}
	if ($attr eq "rbl") {
		check_mailpart($attr, $val);
	}
	if ($attr eq "quotalimit") {
		check_quota($attr, $val);
	}
	if ($attr eq "DefaultQuotaLimit") {
		check_quota($attr, $val);
	}
	if ($attr eq "machine") {
		check_mailpart($attr, $val);
	}
}

# stuff for calling the setuid commands

sub
do_cmd
{
	my $cmd = @_[0];

	$rc = open(FD, $cmd);
	if ($rc <= 0) {
		return(0);
	}
	print(FD "$msgadmindn\n");
	print(FD "$msgadminpw\n");
	$rc = close(FD);
	if (!$rc) {
		return(0);
	}
	return(1);
}

sub
do_rootread
{
	my ($file, $aref) = @_;
	my $tmpfile = "$msgtmp/tmp$$";

	# read into the array.
	$cmd = "|$msgbin/msgroot --read=$file > $tmpfile\n";
	$ret = do_cmd($cmd);
	if ($ret == 0) {
		return($ret);
	}
	# go get our file contents
	if (!open(FD, $tmpfile)) {
		# Unable to open temporary file
		print STDERR msgi18nsmsg(11, 1, "$tmpfile", "$@");
		exit(1);
	}
	@$aref = <FD>;
	close(FD);
	unlink($tmpfile);
	return(1);
}

sub
do_rootwrite
{
	my ($file, $aref) = @_;
	my $tmpfile = "$msgtmp/tmp$$";

	# output the array to the tmpfile
	if (!open(FD, ">$tmpfile")) {
		# Unable to write file $tmpfile: $@\n
		print STDERR msgi18nsmsg(22, 1, "$tmpfile", "$@");
		exit(1);
	}
	foreach $line (@$aref) {
		print FD $line;
	}
	close(FD);

	# send it
	$cmd = "|$msgbin/msgroot --write=$file --from=$tmpfile";
	$ret = do_cmd($cmd);
	if ($ret == 0) {
		# Unable to write file
		print STDERR msgi18nsmsg(22, 1, "$file", "$@");
		unlink($tmpfile);
		exit(1);
	}
	unlink($tmpfile);
	return(1);
}

sub
do_rootserviceaction
{
	my ($service, $action) = @_;

	# reload
	$cmd = "|$msgbin/msgroot --service=$service --action=$action";
	$ret = do_cmd($cmd);
	if ($ret == 0) {
		# service action $service $action failed
		die msgi18nsmsg(23, 1, $service, $action);
	}
	return(1);
}

sub
do_rootbackupdb
{

	# Backup the cyrus mailboxes.db
	$cmd = "|$msgbin/msgroot --backupdb";
	$ret = do_cmd($cmd);
	if ($ret == 0) {
		# Backup of mailboxes.db failed
		die msgi18nsmsg(24, 1);
	}
	return(1);
}

#
# filter builder for ldap queries to escape out special characters
# pass in the filter with %1, %2 prototypes in arg[0], and the successive
# arguments arg[1]... will be escaped and sustituted into the filter.
#
sub
fbuild
{
	$args = scalar(@_) - 1;
	$filter = $_[0];

	for ($index = $args; $index > 0; $index--) {
		$sub = "%$index";
		$arg = $_[$index];

		if (index($filter, $sub) < 0) {
			# "Internal error: string $_[0] does not contain $sub
			die msgi18nsmsg(25, 1, $_[0], $sub);
		}

		$arg =~ s/\\/\\\\/g;
		$arg =~ s/\*/\\*/g;
		$arg =~ s/\(/\\(/g;
		$arg =~ s/\)/\\)/g;
		$arg =~ s/\./\\./g;

		$filter =~ s/$sub/$arg/;
	}
	return($filter);
}

# check for duplicate array entry in a case insensitive manner.
sub
IsPresent
{
	my ($search, @list) = @_;

	$search =~ tr/A-Z/a-z/;
	foreach $item (@list) {
		$item =~ tr/A-Z/a-z/;
		if ($item eq $search) {
			return 1;
		}
	}
	return 0;
}

# identify the primary domain by where the administrator user lives.
sub
getmasterdomain
{
	my @tmp = split(',', $msgadmindn);
	my $masterdomain = substr($tmp[2], 3, 9999);
	return $masterdomain;
}

sub
getcharset
{
	my ($lang) = @_;
	my ($charset, $line, @lines, @arr, @charsets, $item);

	$charset = "ISO-8859-1";
	@arr = split('\.', $lang);
	if (scalar(@arr) < 2) {
		return($charset);
	}
	# now search for if this is a valid charset.
	# if iconv not there use unvalidated charset.
	$charset = $arr[1];
	if (! (-f "/usr/bin/iconv")) {
		return($charset);
	}
	open(FD, "/usr/bin/iconv --list|");
	@lines = <FD>;
	close(FD);
	@charsets = ();
	foreach $line (@lines) {
		chomp($line);
		if (length($line) == 0) {
			next;
		}
		if (substr($line, 0, 1) ne " ") {
			next;
		}
		@arr = split("[ ,\n]+", $line);
		foreach $item (@arr) {
			if ($item ne "") {
				push(@charsets, $item);
			}
		}
	}
	foreach $item (@charsets) {
		if ($charset =~ /$item/i) {
			$charset = $item;
			last;
		}
	}
	return($charset);
}

# initialize the i18n stuff, open the correct catalog (based on LANG variable)
# pass in the set number.  Name of catalog is always msgutil.cat.
# if we cannot find a catalog for the given language we fall back
# to the default catalog.
sub
msgi18ninit
{
	my ($set) = @_;
	my ($charset, $lang);

	# use our hard coded NLSPATH to prevent security problems
	$ENV{NLSPATH} = $msgi18n_nlspath;

	# these are the global variables
	$msgcat = new Locale::Msgcat;
	$msgset = $set;

	# determine character set
	$lang = $ENV{"LANG"};
	$charset = getcharset($lang);

	# first try to open the catalog in the system location
	my $rc = $msgcat->catopen($msgi18nfile, 0);
	my $msg = $msgcat->catgets(1, 1, "nofind");
	# catalog not found, try the default catalog (english).
	if ($msg eq "" || $msg eq "nofind") {
		$msgcat->catclose();
		my $catfile = File::Basename::dirname($0);
		$catfile = "$catfile/$msgi18nfile";
		$rc = $msgcat->catopen($catfile, 0);
		$msg = $msgcat->catgets(1, 1, "nofind");
		if ($msg eq "" || $msg eq "nofind") {
			print STDERR "ERROR:256:Message catalog ($msgi18nfile) not found\n";
			exit(1);
		}
	}

	if ($msgi18n_tx_to_converter == 0) {
		$msgi18n_tx_to_converter = Text::Iconv->new($charset, "UTF-8");
	}
	if ($msgi18n_tx_from_converter == 0) {
		$msgi18n_tx_from_converter = Text::Iconv->new("UTF-8", $charset);
	}
}

# get a message from default message set and substitute the arguments
# if the message does not exist in the set we die.
sub
msgi18nmsg
{
	my ($msgnum) = @_;

	my $msg = $msgcat->catgets($msgset, $msgnum, "nofind");
	if ($msg eq "nofind") {
		print(STDERR "ERROR:256:Missing message in message catalog: set $msgset message $msgnum\n");
		exit(1);
		
	}
	$msg = msgi18n_tx_from_utf8($msg);

	my $args = scalar(@_) - 1;
	my $arg;
	my $index;

	for ($index = $args; $index > 0; $index--) {
		my $sub = "%$index";
		$arg = $_[$index];

		if (index($msg, $sub) < 0) {
			if ($msgi18ndebug == 1) {
				die("ERROR:256:Internal error: message does not contain $sub: set $msgset message $msgnum");
			}
		} else {
			$msg =~ s/$sub/$arg/;
		}
	}
	# check that no %digits remain
	if ($msg =~ /%[0-9]/) {
		die("ERROR:256:Internal error: message has unused arguments: $msg");
	}
	return($msg);
}

# get a message from the set as specified and substitute the arguments
# if the message does not exist in the set we die.
sub
msgi18nsmsg
{
	my ($msgnum, $locset) = @_;

	my $msg = $msgcat->catgets($locset, $msgnum, "nofind");
	if ($msg eq "nofind") {
		print(STDERR "ERROR:256:Missing message in message catalog: set $locset message $msgnum\n");
		exit(1);
		
	}
	$msg = msgi18n_tx_from_utf8($msg);

	my $args = scalar(@_) - 2;
	my $arg;
	my $index;

	for ($index = $args; $index > 0; $index--) {
		my $sub = "%$index";
		$arg = $_[$index + 1];

		if (index($msg, $sub) < 0) {
			if ($msgi18ndebug == 1) {
				die("ERROR:256:Internal error: message does not contain $sub: set $locset msgnum $msgnum");
			}
		} else {
			$msg =~ s/$sub/$arg/;
		}
	}
	# check that no %digits remain
	if ($msg =~ /%[0-9]/) {
		if ($msgi18ndebug == 1) {
			die("ERROR:256:Internal error: message has unused arguments: $msg");
		}
	}
	return($msg);
}

# Used to see if there is a message in the default message set.
sub
msgi18nmsgck
{
	my ($msgnum) = @_;

	my $msg = $msgcat->catgets($msgset, $msgnum, "nofind");
	if ($msg eq "nofind") { $msg = ""; }
	return($msg);
}

# Used to see if there is a message in the specified set
sub
msgi18nsmsgck
{
	my ($msgnum, $locset) = @_;

	my $msg = $msgcat->catgets($locset, $msgnum, "nofind");
	if ($msg eq "nofind") { $msg = ""; }
	return($msg);
}

# our i18n character set translation routines.
# basically in and out of UTF8 to the external native character set
# for now we assume that ISO-8859-1 is the external character set.
# this is convenient as all 8-bit values are essentially preserved.

sub
msgi18n_tx_to_utf8
{
	my ($msg) = @_;

	$nmsg = $msgi18n_tx_to_converter->convert($msg);
	return($nmsg);
}

sub
msgi18n_tx_from_utf8
{
	my ($msg) = @_;

	$nmsg = $msgi18n_tx_from_converter->convert($msg);
	return($nmsg);
}

sub
msgi18naction
{
	my $msg;

	$msgi18naction{"start"} = msgi18nsmsg(30, 1);
	$msgi18naction{"stop"} = msgi18nsmsg(31, 1);
	$msgi18naction{"restart"} = msgi18nsmsg(32, 1);
	$msgi18naction{"reload"} = msgi18nsmsg(33, 1);

	$msgservices{"steltor_cts"}->{"description"} = msgi18nsmsg(34, 1);
	$msgservices{"ldap"}->{"description"} = msgi18nsmsg(35, 1);
	$msgservices{"postfix"}->{"description"} = msgi18nsmsg(36, 1);
	$msgservices{"cyrus"}->{"description"} = msgi18nsmsg(37, 1);
	$msgservices{"docview"}->{"description"} = msgi18nsmsg(38, 1);

}

#
# Steltor
# #
# boolean function determines if Steltor Calendar is installed locally
# by checking to see if the utility to manage calendar users is present.
sub
steltor_installed
{
	if ( -e "/users/unison" ) {
        	return(1);
        } else {
                return(0);
	}
}
#
# Steltor
# 
# returns a hash reference that contains a list of the Steltor 
# calendar node numbers (nodeids) and their matching alias name.
sub
getCtsNodeList
{
	if ( steltor_installed ) {
		# if not defined get path to the VMS bin directory
		if (! $msgbin ) {
			parse_conf("noauth");
		}
		# get node list
		@nodes = split ' ', `$msgbin/msgprofilget -n`;
		foreach $nodeNum ( @nodes ) {
		        $nodeAlias = `$msgbin/msgprofilget -a $nodeNum`;
			#if ( $nodeAlias eq "\n" ) { $nodeAlias = $nodeNum; }
			$ctsNodeList{$nodeNum} = $nodeAlias;
		}
	} 
	return %ctsNodeList;
}

# random password generator.
sub
mkpwd
{
	my ($x, $y, $y, $pwd);

	$x = rand();
	if ($x < .5) {
		$x = $x + .5;
	}
	$y = time();
	$z = abs(int($x * $y));
	$pwd = "x${z}x";
	return($pwd);
}

return 1;	# required
