#!/usr/bin/perl

BEGIN {
	unshift(@INC, "/opt/lsb-caldera.com-volution/msg/bin");
}

use msglib;


#-----------------------------------------------------------------------------
# Three states for parsing.
# NOTFOUND  - we haven't found what we're looking for
# FOUND     - we've found what we're looking for and we're still finding it
# INSTALLED - we've found what we're looking for and we're done with modifying
#-----------------------------------------------------------------------------
my $NOTFOUND=0;
my $FOUND=1;
my $INSTALLED=2;

#-----------------------------------------------------------------------------
# Three variables associated with httpd.conf directives
#  	inloadmod - the LoadModule directive
#	inaddmod  - the AddModule directive
#	vmsmod    - the configuration for VMSG specific directories
#-----------------------------------------------------------------------------
my $inloadmod=$NOTFOUND;
my $inaddmod=$NOTFOUND;
my $vmsmod=$NOTFOUND;

my $modsrc="/opt/lsb-caldera.com-volution/msg/bin/mod_auth_ldap.so";
my $moddest="/usr/lib/apache/mod_auth_ldap.so";
my $htconf="/etc/httpd/httpd.conf";

# Install the mod_auth_ldap Apache module
if ( -f $moddest ) 
{
	print "mod_pam_ldap already installed\n";
}
else
{
	system("cp -f $modsrc $moddest");
}


# Use parse_conf to load the msg.conf information.
# Setting msgadminpw to something causes parse_conf to not 
# prompt for a password (which we don't want).
$msgadminpw="XXX";
parse_conf;

# Derive the default LDAP port from the global variables returned by
# parse_conf.
my $ldapport=389;
$_ = $ldaphost;
if (/.*:.*/) {
	($ldaphost, $ldapport) = split(":");
}

# Begin modifying httpd.conf
open(OLD, "$htconf") || die "$htconf:$!";
open(NEW, ">$htconf.bak") || die "$htconf.bak:$!";

# First check to see if it is already modified
# If it is, bail out.
while(<OLD>) 
{
	if (/.*mod_auth_ldap.*/ && ! /^#/ ) 
	{
		print STDERR "mod_auth_ldap already configured\n";
		$inloadmod=$INSTALLED;
		$inaddmod=$INSTALLED;
	}
}

seek(OLD, 0, 0);
while(<OLD>) 
{
	if (/^LoadModule.*/) 
	{
		if ( $inloadmod == $NOTFOUND ) {
			$inloadmod=$FOUND;
		}
	}
	elsif (/^AddModule.*/) 
	{
		if ( $inaddmod == $NOTFOUND ) {
			$inaddmod=$FOUND;
		}
	}
	elsif (/^# END VOLUTION MESSAGING SERVER CONFIGURATION/) {
		$vmsmod=$INSTALLED;
		next;
	}
	elsif (/^# START VOLUTION MESSAGING SERVER CONFIGURATION - DO NOT MODIFY/) 
	{

print NEW '# START VOLUTION MESSAGING SERVER CONFIGURATION - DO NOT MODIFY
Alias /msg "/opt/lsb-caldera.com-volution/msg/webgui/"
<Directory "/opt/lsb-caldera.com-volution/msg/webgui">
        Options Indexes MultiViews ExecCGI
        AllowOverride None Limit FileInfo Options
        Order allow,deny
        Allow from all
</Directory>
Alias /msghelp "/usr/share/doc/OpenLinux/"
<Directory "/usr/share/doc/OpenLinux">
        Options Indexes MultiViews
        AllowOverride None
        Order allow,deny
        Allow from all
</Directory>
Alias /calendar "/home/httpd/html/calendar/"
Alias /pub/calendar "/home/httpd/html/calendar/"
<Directory "/home/httpd/html/calendar">
	Dav on
	Options Indexes MultiViews
	order allow,deny
	allow from all
	AuthName "Calendar"
	AuthType Basic
	require valid-user
';
print NEW "	LDAP_Server $ldaphost
	LDAP_Port   $ldapport
	Base_DN     \"$ldapsuffix\"
	Bind_DN     \"$msgdn\"
	Bind_Pass   $msgpw
";
print NEW 'UID_Attr    uid
</Directory>
# END VOLUTION MESSAGING SERVER CONFIGURATION
';
		$vmsmod=$FOUND;
		next;
	}
	else 
	{
		if ( $inloadmod == $FOUND ) {
			print NEW "LoadModule ldap_auth_module   lib/apache/mod_auth_ldap.so\n";
			$inloadmod=$INSTALLED;
		}
		if ( $inaddmod == $FOUND ) {
			print NEW "AddModule mod_auth_ldap.c\n";
			$inaddmod=$INSTALLED;
		}
		if ( $vmsmod == $FOUND ) {
			next;
		}
	}
	print NEW $_;
}
close(OLD);
close(NEW);
rename("$htconf.bak", "$htconf");

# Reload httpd to suck in the changes.
system("/etc/rc.d/init.d/httpd", "stop");
system("/etc/rc.d/init.d/httpd", "start") && die "httpd: could not start\n";


# Configure the free/busy calendar directory for Web Folders 
# support.
system("/bin/mkdir /home/ftp/pub/calendar/.DAV");
system("/bin/chmod 770 /home/ftp/pub/calendar/.DAV");

# Resolve permission conflicts between ftp and httpd.  Anon FTP
# will now share the httpd uid as will all anonymous files.
$ftpid = getpwnam("httpd");
system("/usr/sbin/usermod -u $ftpid -o ftp");
system("/bin/chown -R ftp /home/ftp/pub");
exit(0);
