<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

/******************************************************************************
 * class ErrorStack
 *
 * Handles the various error tracking required by the GUI. This includes 
 * keeping an error stack ($stack), returning the stack as a string (unwind())
 * and finding the last pushed error number (from the backend) so that the
 * GUI can decide which course of error recovery can be taken.
 ******************************************************************************
 */
class ErrorStack {

	var $stack;

	/**********************************************************************
	 * constructor
	 *
	 * no action is required
	 *
	 * Returns an ErrorStack obj reference
         **********************************************************************
	 */
	function ErrorStack() {
		;
	}

	/**********************************************************************
	 * function push()
	 *
	 * Pushes the give message onto the stack array
	 *
	 * Parameters:
	 * o $msg - string - the string (localized or otherwise) to add to
	 *   the stack.
 	 * Returns:
	 * o nothing
         **********************************************************************
	 */
	function push($msg) {
		array_push($this->stack, $msg);
	}

	/**********************************************************************
	 * function pop()
	 *
	 * Pops the last error message off the stack array
	 *
	 * Parameters:
	 * o none
	 * Returns:
	 * o string (localized or otherwise) which was last added to the stack
         **********************************************************************
	 */
	function pop() {
		return array_pop($this->stack);
	}

	/**********************************************************************
	 * function clear()
	 *
	 * Unsets the internal array used for the stack
	 *
	 * Parameters:
	 * o none
	 * Returns:
	 * o nothing
         **********************************************************************
	 */
	function clear() {
		unset($this->stack);
	}

	/**********************************************************************
	 * function lastErrorNo()
	 *
	 * Searchs the stack to find the last entry that contains an error
	 * number. Error numbers come from the backend and help the GUI
	 * recover gracefully from error conditions 
	 *
	 * Parameters:
	 * o none
	 * Returns:
	 * o the last error number or false if none is found.
         **********************************************************************
	 */
	function lastErrorNo() {
		;
	}

	/**********************************************************************
	 * function unwind()
	 *
	 * This goes through the stack popping off messages and concatenating
	 * them in to a long, newline separated string. The stack is left
	 * empty.
	 *
	 * NOTE: this should probably be rewritten to use a join function
	 *
	 * Parameters:
	 * o none:
	 * Returns:
	 * o string containing all the messages of the stack
         **********************************************************************
	 */
	function unwind() {

		// Too costly?
		//$retval = '';
		//while ($this->stack) {
		//	$retval .= array_pop($this->stack) . "\n";
		//}

		$retval = join("\n", $this->stack);
		unset($this->stack);

		return $retval;
	}
}

/******************************************************************************
 * Function stdErrorMsg()
 *
 * This function is deprecated but still appears in the GUI code. It formats
 * error message array and an exit value into a string to be displayed to 
 * the user.
 ******************************************************************************
 */
function stdErrorMsg($errors, $exit) {
	$newline = getIntlMsg('MSG_NEWLINE');

	if (is_array($errors)) {
		$retval = join($newline, $errors) . $newline;
   	} else {
		$retval = $errors . $newline;
   	}

   	if ($exit !== '') {
		$retval .= $newline . 
			getIntlMsg('ERR_COMMAND_EXIT_VALUE', $exit) .
			$newline;
	}

	return $retval;
}

?>
