<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

class UserSelectList extends ObjectTable {

	function UserSelectList($varArray, $id) {
		global $session_domain;

		$this->list = '';
		$cmd = new Command('msguserlistall');
		$cmd->add_arg('--domain', $session_domain);
		$cmd->add_arg('--uid --fullname --mail');
		$cmd->run();
		$lines = $cmd->getOutput();

		unset($this->list);

		// Populate the list 
		if ($lines != '') foreach ($lines as $line) {

			$line = trim($line);

			if ($line == '') {
				if ($user != '') {
					$this->list[] = $user;
					$user = '';
				}
				continue;
			}

			if ($user == '' && $line != '') {
				$user = new User();
			}

			list($attr, $value) = split(':', $line, 2);
			$user->setAttr($attr, $value);
		}

		// Set parameters
		$this->linkactionfn = 'RequestPage';
		
		// Apply filter if any
		$varArray['listClass'] = 'UserSelectList';
		$this->setFilter($varArray, $id);
		if (isset($this->filter)) $this->applyFilter($this->filter);

		// Setup the table
		if (Table::isActive($varArray, $id)) {
			$table = Table::getActive();
			$table->setParams($varArray, $this->count());
		} else {
			$table = new Table($id);

			$name = getIntlMsg('MSG_USERS_HEADING_EMAIL');
			$table->addDataColumn($name, Column::ASCEND(), 'mail');

			$name = getIntlMsg('MSG_USERS_HEADING_NAME');
			$table->addDataColumn($name, Column::ASCEND(), 'fullname');

			$name = getIntlMsg('MSG_USERS_HEADING_ADD');
			$table->addDisplayColumn($name, 
				"return UserAdd('%%ROWID%%')");

			$table->setParams($HTTP_GET_VARS, $this->count());
			$table->rows = 10;
			$table->setSortedColumn($table->firstSortedColumn());
		}

		// Apply sorting
		$attr = $table->getSortAttrName($table->getSortedColumn());
		$dir  = $table->getColumnSortDir($table->getSortedColumn());
		$this->sort(User::comparisonFunctionName($attr ,$dir));

		// Save table
		$this->table = $table;
		$this->table->save();
	}

	function getListValue($index, $col = '') {
		global $session_lang;

		// if now $col then return the row value (unique id)
		if ($col === '') return $this->list[$index]->getMail();

		switch ($col) {
			case '0':
			case 'mail':
				$retval = $this->list[$index]->shortmail();
				break;
			case '1' :
			case 'fullname':
				$retval = $this->list[$index]->getFullName();
				break;
			case '2' :
				$retval = $this->table->getTemplate('addIcon');
				$retval = ereg_replace('%%LANG%%', $session_lang, $retval);
				$retval = stripslashes($retval);
				break;
			default:
				$retval = "";
				break;
		}

		return $retval;
	}

	/**********************************************************************
	 * class function getSearchAttrs()
	 *
	 * return an array containing the User attributes that can be used
	 * to filter the results displayed in the table. The array has the
	 * internally meaningful attribute as the key and the displayable
	 * attribute name as the value;
	 **********************************************************************
	 */
	function getSearchAttrs() {
		return array(
			'mail' => getIntlMsg('MSG_USERS_HEADING_EMAIL'),
			'fullname' => getIntlMsg('MSG_USERS_HEADING_NAME')
		);
	}

}

?>
